import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Collections;
import java.util.List;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;


public class ListSorter implements ActionListener
{

	private final List<Integer> list_;
	private final JPanel panel_;
	private final Lock lock_;
	private final JButton shuffleButton_;
	private final JButton sortButton_;

	public ListSorter(List<Integer> list, JPanel panel, JButton sortButton, JButton shuffleButton)
	{
		list_ = list;
		panel_ = panel;
		lock_ = new ReentrantLock();
		sortButton_ = sortButton;
		shuffleButton_ = shuffleButton;
	}

	@Override
	public void actionPerformed(ActionEvent e)
	{
		if(e.getSource() == sortButton_)
		{
			new Thread(new Runnable()
			{
				@Override
				public void run()
				{
					if(lock_.tryLock())
					{
						quicksort(list_, 0, list_.size() - 1);
						lock_.unlock();
					}
				}
			}).start();
		}
		else if(e.getSource() == shuffleButton_)
		{

			if(lock_.tryLock())
			{
				Collections.shuffle(list_);
				panel_.repaint();
				lock_.unlock();
			}
		}


	}

	//Quicksort the array
	private void quicksort(List<Integer> array, int left, int right)
	{
		if(left >= right)
			return;

		int index = partition(array, left, right);
		quicksort(array, left, index - 1);
		quicksort(array, index + 1, right);
	}

	//Partition the array into two halves and return the
	//index about which the array is partitioned
	private int partition(List<Integer> array, int left, int right)
	{
		//Makes the leftmost element a good pivot,
		//specifically the median of medians
		findMedianOfMedians(array, left, right);
		int pivotIndex = left, pivotValue = array.get(pivotIndex), index = left, i;

		swap(array, pivotIndex, right);
		for(i = left; i < right; i++)
		{
			if(array.get(i) < pivotValue)
			{
				swap(array, i, index);
				index += 1;
			}
		}
		swap(array, right, index);

		return index;
	}

	//Computes the median of each group of 5 elements and stores
	//it as the first element of the group. Recursively does this
	//till there is only one group and hence only one Median
	private int findMedianOfMedians(List<Integer> array, int left, int right)
	{
		if(left == right)
			return array.get(left);

		int i, shift = 1;
		while(shift <= (right - left))
		{
			for(i = left; i <= right; i+=shift*5)
			{
				int endIndex = (i + shift*5 - 1 < right) ? i + shift*5 - 1 : right;
				int medianIndex = findMedianIndex(array, i, endIndex, shift);

				swap(array, i, medianIndex);
			}
			shift *= 5;
		}

		return array.get(left);
	}

	//Find the index of the Median of the elements
	//of array that occur at every "shift" positions.
	private int findMedianIndex(List<Integer> list, int left, int right, int shift)
	{
		int i, groups = (right - left)/shift + 1, k = left + groups/2*shift;
		for(i = left; i <= k; i+= shift)
		{
			int minIndex = i, minValue = list.get(minIndex), j;
			for(j = i; j <= right; j+=shift)
				if(list.get(j) < minValue)
				{
					minIndex = j;
					minValue = list.get(minIndex);
				}
			swap(list, i, minIndex);
		}

		return k;
	}

	//Swap integer values by list indexes
	void swap(List<Integer> list, int a, int b)
	{
		int temp = list.get(a);
		list.set(a, list.get(b));
		list.set(b, temp);
		try
		{
			Thread.sleep(10);
			SwingUtilities.invokeLater(new Runnable()
			{
				public void run()
				{
					panel_.repaint();
				}
			});
		}
		catch (InterruptedException e)
		{
			e.printStackTrace();
		}

	}



}
